
; Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
;
;
;  Redistribution and use in source and binary forms, with or without
;  modification, are permitted provided that the following conditions
;  are met:
;
;    Redistributions of source code must retain the above copyright
;    notice, this list of conditions and the following disclaimer.
;
;    Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the
;    distribution.
;
;    Neither the name of Texas Instruments Incorporated nor the names of
;    its contributors may be used to endorse or promote products derived
;    from this software without specific prior written permission.
;
;  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
;  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
;  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
;  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
;  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
;  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
;  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
;  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
;  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
;  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;


;******************************************************************************
;	(C) Copyright 2016, Texas Instruments, Inc
;	
;
;	Version		Description					Author
; 	0.1     	Created		                Thomas Leyrer
;
	.if ! $defined(__sorte_master_cfg_inc)
	.define	"1", __sorte_master_cfg_inc

;------------------------------------------------------------------------------
;                       Device Configuration
;------------------------------------------------------------------------------
PRU_FREQ            .set    4       ; 250 MHz -> 4ns
;BRIDGE_DELAY_CONST	.set	84	    ; brige delay: 168ns = signals RX_DV to TX_EN; RX to TX timestamps = 84ns
BRIDGE_DELAY_LINEDELAY .set 320     ; 0x3A in TXCFG START DELAY - 320ns
BRIDGE_DELAY_SYNC_IOEX .set 160     ; 0x14 in TXCFG START DELAY - 160ns ; (80ns + 2x40 L0 FIFO); measured jitter from 156 - 168ns
TTS_GUARD_DELAY     .set    35
IEP_RW_ACCESS_CONST	.set	15		; read/write access to IEP timer

CYCLE_TIME_10US		.set	10000	; 10 usec cuycle time value

IEP_DEFAULT_INCREMENT       .set    4
IEP_FAST_INCREMENT          .set    (IEP_DEFAULT_INCREMENT + 1)
IEP_SLOW_INCREMENT          .set    (IEP_DEFAULT_INCREMENT - 1)


; Aproximate Moving Average Filter
AMA_N_DEPTH         .set    10       ; filter depth 2^N
;   2 -> N = 4
;   6 -> N = 64
;   8 -> N = 256
;   10 -> N = 1024

; Simple Moving Average filter
SMA_WRAP_AROUND_1	.set	0x03	; 1 SMA values
SMA_MULTIPLIER_1	.set	0		; /1 SMA values
SMA_WRAP_AROUND_2	.set	0x07	; 2 SMA values
SMA_MULTIPLIER_2	.set	1		; /2 SMA values
SMA_WRAP_AROUND_4	.set	0x0f	; 4 SMA values
SMA_MULTIPLIER_4	.set	2		; /4 SMA values
SMA_WRAP_AROUND_8	.set	0x1f	; 8 SMA values
SMA_MULTIPLIER_8	.set	3		; /8 SMA values
SMA_WRAP_AROUND_16	.set	0x3f	; 16 SMA values
SMA_MULTIPLIER_16	.set	4		; /16 SMA values
SMA_WRAP_AROUND_32	.set	0x7F	; 32 SMA values
SMA_MULTIPLIER_32	.set	5		; /32 SMA values
SMA_WRAP_AROUND_64	.set	0xFF	; 64 SMA values
SMA_MULTIPLIER_64	.set	6		; /64 SMA values




SMA_WRAP_AROUND		.set	SMA_WRAP_AROUND_64
SMA_MULTIPLIER		.set	SMA_MULTIPLIER_64


FILTER_CTRL_SMA_0_BIT	.set 	0
FILTER_CTRL_SMA_8_BIT	.set 	1
FILTER_CTRL_SMA_64_BIT	.set 	2

FILTER_CTRL_RESET_BIT 	.set	7
FILTER_CTRL_NO_COMP		.set	6
FILTER_CTRL_NO_COMP_AT_STARTUP .set 5

;line delay measurment cycles
LD_SHIFT_64         .set    6
LD_SHIFT_16			.set	4
LD_SHIFT_8			.set	3

; event configuration
PRU2ARM_LINK                    .set    25      ; link change event
PRU2PRU_EVT1					.set	26
PRU2PRU_EVT2					.set	27
PRU2PRU_INTR1					.set	(0x20 | (PRU2PRU_EVT1-16))
PRU2PRU_INTR2					.set	(0x20 | (PRU2PRU_EVT2-16))
PRU2ARM_INTR_LINK               .set    (0x20 | (PRU2ARM_LINK-16))

;------------------------------------------------------------------------------
;                       State Machine Configuration
;------------------------------------------------------------------------------
	
; default state machine parameters
DISC_REPEAT		.set		3
PARAM_REPEAT	.set		3
SYNC_REPEAT		.set		100
LINED_BURST		.set		5
LINED_CYCLE		.set		8000	; in ms
	

;------------------------------------------------------------------------------
;                       Ethernet Parameters
;------------------------------------------------------------------------------

	;define		MIN_IPG			100 ; *ocp_clk -> 5ns = 500 ns
	;define		PREAMBLE_SIZE	2
	;define     CRC32			0
	;define 	CRC16			1
	;define		FCS_TYPE		CRC32 
	
;------------------------------------------------------------------------------
;                       Application Parameters
;------------------------------------------------------------------------------

	;define		IOEX_CYCLE			25000	; in ns
	;define		MAX_INPUT_DATA		8192
	;define		MAX_OUTPUT_DATA		8192
	;define		T_IN				1000
	;define		T_OUT				1000

;------------------------------------------------------------------------------
;                       Application Parameters
;------------------------------------------------------------------------------
INPUT_BYTES_COUNT	.set		12
OUTPUT_BYTES_COUNT	.set		12

;------------------------------------------------------------------------------
;                       PRU0 Data RAM defines
;------------------------------------------------------------------------------

; use first 256 bytes for configuration and status registers
SORTE_REG_INTERFACE	.set	0x0
CRC8_TABLE_OFFESET	.set 	0x100
DISCOV_RX_OFFSET	.set	0x200
OUT_DATA_OFFSET		.set 	0x700
IN_DATA_OFFSET		.set	0x800

	.endif
